﻿/*
  This file is part of Sanmill.
  Copyright (C) 2019-2021 The Sanmill developers (see AUTHORS file)

  Sanmill is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Sanmill is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pieceitem.h"
#include "graphicsconst.h"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QStyleOption>

PieceItem::PieceItem(QGraphicsItem *parent) :
    QGraphicsItem(parent),
    num(0)
{
    Q_UNUSED(parent)
        setFlags(ItemIsSelectable
                 // | ItemIsMovable
        );

    setCacheMode(DeviceCoordinateCache);

    setCursor(Qt::OpenHandCursor);

    //setAcceptedMouseButtons(Qt::LeftButton);

    setAcceptedMouseButtons(nullptr);
    //setAcceptHoverEvents(true);

    model = Models::noPiece;

    size = PIECE_SIZE;

    selectLineWeight = LINE_WEIGHT;

    removeLineWeight = LINE_WEIGHT * 5;

#ifdef MOBILE_APP_UI
    selectLineColor = Qt::gray;
#else
    selectLineColor = Qt::darkYellow;
#endif /* MOBILE_APP_UI */

    removeLineColor = QColor(227, 23, 13);
    removeLineColor.setAlphaF(0.9);
}

PieceItem::~PieceItem() = default;

QRectF PieceItem::boundingRect() const
{
    return QRectF(-size / 2, -size / 2, size, size);
}

QPainterPath PieceItem::shape() const
{
    QPainterPath path;
    path.addEllipse(boundingRect());
    return path;
}

void PieceItem::paint(QPainter *painter,
                      const QStyleOptionGraphicsItem *option,
                      QWidget *widget)
{
    Q_UNUSED(option)
        Q_UNUSED(widget)

    // Empty models don't draw pieces

    switch (model) {
    case Models::whitePiece:
        // If the model is white, draw white pieces
#ifdef MOBILE_APP_UI
        painter->setPen(Qt::NoPen);
        painter->setBrush(QColor(0, 93, 172));
        painter->drawEllipse(-size / 2, -size / 2, size, size);
#else
        painter->drawPixmap(-size / 2, -size / 2, size, size,
            QPixmap(":/image/resources/image/white_piece.png"));
#endif /* MOBILE_APP_UI */
        break;

    case Models::blackPiece:
        // If the model is black, draw black pieces
#ifdef MOBILE_APP_UI
        painter->setPen(Qt::NoPen);
        painter->setBrush(QColor(231, 36, 46));
        painter->drawEllipse(-size / 2, -size / 2, size, size);
#else
        painter->drawPixmap(-size / 2, -size / 2, size, size,
            QPixmap(":/image/resources/image/black_piece.png"));
#endif /* MOBILE_APP_UI */
        break;
    default:
        break;
    }

    // If the model requires the serial number to be displayed
    if (showNum) {
        if (model == Models::whitePiece)
            painter->setPen(QColor(255, 255, 255));

        if (model == Models::blackPiece)
            painter->setPen(QColor(0, 0, 0));

        QFont font;
        font.setFamily("Arial");
        font.setPointSize(size / 3);
        painter->setFont(font);

        painter->drawText(boundingRect().adjusted(0, 0, 0, -size / 12),
                            Qt::AlignCenter, QString::number(num));

    }

    // If the model is selected, draw four small right angles
    if (isSelected()) {
        QPen pen(selectLineColor, selectLineWeight, Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin);
        painter->setPen(pen);
        const int xy = (size - selectLineWeight) / 2;

        painter->drawLine(-xy, -xy, -xy, -xy / 2);
        painter->drawLine(-xy, -xy, -xy / 2, -xy);
        painter->drawLine(xy, -xy, xy, -xy / 2);
        painter->drawLine(xy, -xy, xy / 2, -xy);
        painter->drawLine(xy, xy, xy, xy / 2);
        painter->drawLine(xy, xy, xy / 2, xy);
        painter->drawLine(-xy, xy, -xy, xy / 2);
        painter->drawLine(-xy, xy, -xy / 2, xy);
    }

    // If the model is deleted, cross it
    if (deleted) {
        QPen pen(removeLineColor, removeLineWeight, Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin);
        painter->setPen(pen);

        painter->drawLine(-size / 3, -size / 3, size / 3, size / 3);
        painter->drawLine(size / 3, -size / 3, -size / 3, size / 3);
    }
}

void PieceItem::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    // When the mouse is pressed, it becomes the shape of the hand it holds
    setCursor(Qt::ClosedHandCursor);
    QGraphicsItem::mousePressEvent(mouseEvent);
}

void PieceItem::mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    QGraphicsItem::mouseMoveEvent(mouseEvent);
}

void PieceItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    // When the mouse is released, it becomes an extended hand
    setCursor(Qt::OpenHandCursor);
    QGraphicsItem::mouseReleaseEvent(mouseEvent);
}
